@@ -1,6 +1,7 @@
 import base64
 import io
 import json
+import numpy as np
 import requests
 import time
 import uuid
@@ -14,17 +15,30 @@ from qiskit.quantum_info.operators.linear_op import LinearOp
 from urllib.parse import urlencode
 
 
-def serialize_object(object):
+class ComplexEncoder(json.JSONEncoder):
+    def default(self, obj):
+        if isinstance(obj, complex):
+            return {"real": obj.real, "imag": obj.imag}
+        return super().default(obj)
+
+
+def all_numbers(lst):
+    return all(isinstance(x, (int, float, complex)) for x in lst)
+
+
+def serialize_circuit(circuit):
     buffer = io.BytesIO()
-    qpy.dump(object, buffer)
+    qpy.dump(circuit, buffer)
     qpy_binary_data = buffer.getvalue()
-    base64_encoded_object = base64.b64encode(qpy_binary_data).decode("utf-8")
-    return base64_encoded_object
+    base64_encoded_circuit = base64.b64encode(qpy_binary_data).decode("utf-8")
+    return base64_encoded_circuit
+
 
 class AuthenticationFailure(Exception):
     def __init__(self, message):
         self.message = message
 
+
 class Job:
     def __init__(self, job_id):
         self._job_id = job_id
@@ -32,6 +46,7 @@ class Job:
     def id(self):
         return self._job_id
 
+
 class WorkflowJob:
     def __init__(self, job_id):
         self._job_id = job_id
@@ -39,18 +54,34 @@ class WorkflowJob:
     def id(self):
         return self._job_id
 
+
 class InputData:
     def __init__(self, label=None, content=None):
         self.data = {}
         if label:
             self.add_data(label, content)
 
+    def __str__(self):
+        return json.dumps(self.data, indent=4, cls=ComplexEncoder)
+
     def add_data(self, label, content):
         self.check_label(label, self.data)
         try:
             if label == "operator":
-                content = self.validate_and_serialize_operator(content)
-                self.data["operator"] = content
+                operator = content
+                coeffs = None
+                if type(content) == tuple:
+                    operator, coeffs = content
+                sparse_pauli_operator = self.to_sparse_pauli_operator(
+                    operator, coeffs=coeffs
+                )
+                pauli_terms, coefficients = self.serialize_sparse_pauli_operator(
+                    sparse_pauli_operator
+                )
+                self.data["operator"] = {
+                    "pauli_terms": pauli_terms,
+                    "coefficients": coefficients,
+                }
             elif label == "pub":
                 content = self.validate_and_serialize_pub(content)
                 if not "pubs" in self.data.keys():
@@ -128,20 +159,80 @@ class InputData:
                 "The 'paramaters' setting in a PUB must be a list of numbers."
             )
 
-        return (serialize_object(quantum_circuit), paramaters, shots)
+        return (serialize_circuit(quantum_circuit), paramaters, shots)
+
+    def validate_and_serialize_operator(self, operator):
+        if (
+            not isinstance(operator, Operator)
+            and not isinstance(operator, Pauli)
+            and not isinstance(operator, SparsePauliOp)
+            and not (
+                isinstance(operator, tuple)
+                and isinstance(operator[0], PauliList)
+                and isinstance(operator[1], list)
+                and (operator[1] and not all_numbers(operator[1]))
+            )
+        ):
+            raise Exception(
+                "The operator must be an instance of the Operator, Pauli, SparsePauliOp class or a tuple containing a PauliList and a possible empty list of numeric coefficents."
+            )
+
+        if isinstance(operator, Operator):
+            matrix = operator.data
+            if not np.allclose(matrix, matrix.conj().T):
+                print("WARNING: The operator you supplied is not Hermitian!")
+
+        if (
+            isinstance(operator, tuple)
+            and isinstance(operator[0], PauliList)
+            and isinstance(operator[1], list)
+        ):
+            pauli_list = operator[0]
+            coefficients = operator[1]
+            if (
+                coefficients is not None
+                and len(coefficients) > 0
+                and len(pauli_list) != len(coefficients)
+            ):
+                raise Exception(
+                    "The number of Pauli terms in the Pauli list must match the number of coefficients or list of coefficients must be empty."
+                )
+
+        return serialize_circuit(operator)
+
+    def to_sparse_pauli_operator(self, operator, coeffs=None):
+        if isinstance(operator, SparsePauliOp):
+            return operator
+
+        elif isinstance(operator, Pauli):
+            return SparsePauliOp(operator)
+
+        elif isinstance(operator, PauliList):
+            if coeffs is not None and len(coeffs) > 0 and len(coeffs) != len(operator):
+                raise ValueError(
+                    "Number of coefficients must match number of Pauli operators in PauliList"
+                )
+
+            coefficients = (
+                coeffs
+                if (coeffs is not None and len(coeffs) > 0)
+                else [1.0] * len(operator)
+            )
+            pauli_strings = [str(pauli) for pauli in operator]
+            return SparsePauliOp(pauli_strings, coeffs=coefficients)
+
+        elif isinstance(operator, Operator):
+            return SparsePauliOp.from_operator(operator)
+
+    def serialize_sparse_pauli_operator(self, sparse_op):
+        if not isinstance(sparse_op, SparsePauliOp):
+            raise ValueError("Input must be a SparsePauliOp")
+
+        pauli_data = sparse_op.to_list()
+        pauli_terms = [term[0] for term in pauli_data]
+        coefficients = [term[1] for term in pauli_data]
+        return pauli_terms, coefficients
 
-def validate_and_serialize_operator(self, operator):
-    if (
-        not isinstance(operator, Operator)
-        and not isinstance(operator, Pauli)
-        and not isinstance(operator, PauliList)
-        and not isinstance(operator, SparsePauliOp)
-        and not isinstance(operator, LinearOp)
-    ):
-        raise Exception(
-            "The operator must be an instance of the Operator, Pauli, PauliList or SparsePauliOp class."
-        )
-    return self.serialize_object(operator)
 
 class StrafulProvider:
 
@@ -229,7 +320,7 @@ In case the service has been recently started please wait 5 minutes for it to be
         try:
             job_data = {
                 "BackendName": backend,
-                "Circuit": serialize_object(circuit),
+                "Circuit": serialize_circuit(circuit),
                 "Shots": shots,
                 "Comments": comments,
             }
@@ -281,7 +372,9 @@ In case the service has been recently started please wait 5 minutes for it to be
             for input_data_label in input_data.data.keys():
                 input_data_labels.append(input_data_label)
                 content = input_data.data[input_data_label]
-                input_data_items.append(json.dumps(content, indent=4))
+                input_data_items.append(
+                    json.dumps(content, indent=4, cls=ComplexEncoder)
+                )
             job_data = {
                 "BackendName": backend,
                 "WorkflowId": workflow_id,
